@@ -40,6 +40,7 @@ DRAIN_MESSAGE_PAYLOAD = "__TOMODACHI_DRAIN__cdab4416-1727-4603-87c9-0ff8dddf1f22
 MESSAGE_ENVELOPE_DEFAULT = "e6fb6007-cf15-4cfd-af2e-1d1683374e70"
 MESSAGE_PROTOCOL_DEFAULT = MESSAGE_ENVELOPE_DEFAULT  # deprecated
 MESSAGE_TOPIC_PREFIX = "09698c75-832b-470f-8e05-96d2dd8c4853"
+MESSAGE_TOPIC_ATTRIBUTES = "dc6c667f-4c22-4a63-85f6-3ea0c7e2db49"
 FILTER_POLICY_DEFAULT = "7e68632f-3b39-4293-b5a9-16644cf857a5"
 
 AnythingButFilterPolicyValueType = Union[str, int, float, List[str], List[int], List[float], List[Union[int, float]]]
@@ -132,6 +133,8 @@ class AWSSNSSQSTransport(Invoker):
         message_protocol: Any = MESSAGE_ENVELOPE_DEFAULT,  # deprecated
         topic_prefix: Optional[str] = MESSAGE_TOPIC_PREFIX,
         message_attributes: Optional[Dict[str, Any]] = None,
+        topic_attributes: Optional[Union[str, Dict[str, Union[bool, str]]]] = MESSAGE_TOPIC_ATTRIBUTES,
+        overwrite_topic_attributes: bool = False,
         **kwargs: Any,
     ) -> None:
         if message_envelope == MESSAGE_ENVELOPE_DEFAULT and message_protocol != MESSAGE_ENVELOPE_DEFAULT:
@@ -157,7 +160,13 @@ class AWSSNSSQSTransport(Invoker):
                     service, topic, data, message_attributes=message_attributes, **kwargs
                 )
 
-        topic_arn = await cls.create_topic(topic, service.context, topic_prefix)
+        topic_arn = await cls.create_topic(
+            topic,
+            service.context,
+            topic_prefix,
+            attributes=topic_attributes,
+            overwrite_attributes=overwrite_topic_attributes,
+        )
 
         async def _publish_message() -> None:
             await cls.publish_message(topic_arn, payload, cast(Dict, message_attributes), service.context)
@@ -237,7 +246,7 @@ class AWSSNSSQSTransport(Invoker):
         obj: Any,
         context: Dict,
         func: Any,
-        topic: str,
+        topic: Optional[str] = None,
         callback_kwargs: Optional[Union[List, Set, Tuple]] = None,
         competing: Optional[bool] = None,
         queue_name: Optional[str] = None,
@@ -514,7 +523,14 @@ class AWSSNSSQSTransport(Invoker):
             raise AWSSNSSQSConnectionException(error_message, log_level=context.get("log_level")) from e
 
     @classmethod
-    async def create_topic(cls, topic: str, context: Dict, topic_prefix: Optional[str] = MESSAGE_TOPIC_PREFIX) -> str:
+    async def create_topic(
+        cls,
+        topic: str,
+        context: Dict,
+        topic_prefix: Optional[str] = MESSAGE_TOPIC_PREFIX,
+        attributes: Optional[Union[str, Dict[str, Union[bool, str]]]] = MESSAGE_TOPIC_ATTRIBUTES,
+        overwrite_attributes: bool = True,
+    ) -> str:
         if not cls.topics:
             cls.topics = {}
         if cls.topics.get(topic):
@@ -525,12 +541,81 @@ class AWSSNSSQSTransport(Invoker):
         if not connector.get_client("tomodachi.sns"):
             await cls.create_client("sns", context)
 
+        config_base = context.get("options", {}).get("aws_sns_sqs", context.get("options", {}).get("aws", {}))
+        aws_config_base = context.get("options", {}).get("aws", {})
+
+        sns_kms_master_key_id: Optional[str] = None
+        for config_value in (
+            config_base.get("aws_sns_kms_master_key_id"),
+            config_base.get("sns_kms_master_key_id"),
+            aws_config_base.get("aws_sns_kms_master_key_id"),
+            aws_config_base.get("sns_kms_master_key_id"),
+            aws_config_base.get("aws_kms_master_key_id"),
+            aws_config_base.get("kms_master_key_id"),
+            config_base.get("aws_kms_master_key_id"),
+            config_base.get("kms_master_key_id"),
+        ):
+            if config_value is not None:
+                if isinstance(config_value, str) and config_value == "":
+                    sns_kms_master_key_id = ""
+                    break
+                elif isinstance(config_value, bool) and config_value is False:
+                    sns_kms_master_key_id = ""
+                    break
+                elif isinstance(config_value, str) and config_value:
+                    sns_kms_master_key_id = config_value
+                    break
+                else:
+                    raise ValueError(
+                        "Bad value for aws_sns_sqs option sns_kms_master_key_id: {}".format(str(config_value))
+                    )
+
+        topic_attributes: Dict
+
+        if attributes is None or attributes == MESSAGE_TOPIC_ATTRIBUTES:
+            topic_attributes = {}
+        elif not isinstance(attributes, dict):
+            raise Exception(
+                "Argument 'attributes' to 'AWSSNSSQSTransport.create_topic' should be specified as a dict mapping"
+            )
+        else:
+            topic_attributes = copy.deepcopy(attributes)
+
+        if sns_kms_master_key_id is not None and "KmsMasterKeyId" not in topic_attributes:
+            topic_attributes["KmsMasterKeyId"] = sns_kms_master_key_id
+
+        update_attributes = True if topic_attributes else False
+        topic_arn = None
+
         try:
             async with connector("tomodachi.sns", service_name="sns") as client:
                 response = await asyncio.wait_for(
-                    client.create_topic(Name=cls.encode_topic(cls.get_topic_name(topic, context, topic_prefix))),
+                    client.create_topic(
+                        Name=cls.encode_topic(cls.get_topic_name(topic, context, topic_prefix)),
+                        Attributes=topic_attributes,
+                    ),
                     timeout=40,
                 )
+                topic_arn = response.get("TopicArn")
+                update_attributes = False
+
+            if topic_attributes and topic_attributes.get("KmsMasterKeyId") == "":
+                try:
+                    async with connector("tomodachi.sns", service_name="sqs") as client:
+                        topic_attributes_response = await client.get_topic_attributes(TopicArn=topic_arn)
+                        if not topic_attributes_response.get("Attributes", {}).get("KmsMasterKeyId"):
+                            update_attributes = False
+                            overwrite_attributes = False
+                except Exception:
+                    pass
+
+                if overwrite_attributes:
+                    logging.getLogger("transport.aws_sns_sqs").info(
+                        "SNS topic attribute 'KmsMasterKeyId' on SNS topic '{}' will be updated to disable server-side encryption".format(
+                            topic_arn
+                        )
+                    )
+                    update_attributes = True
         except (botocore.exceptions.NoCredentialsError, aiohttp.client_exceptions.ClientOSError) as e:
             error_message = str(e)
             logging.getLogger("transport.aws_sns_sqs").warning(
@@ -543,12 +628,66 @@ class AWSSNSSQSTransport(Invoker):
             asyncio.TimeoutError,
         ) as e:
             error_message = str(e) if not isinstance(e, asyncio.TimeoutError) else "Network timeout"
+            if "Topic already exists with different attributes" in error_message:
+                try:
+                    async with connector("tomodachi.sns", service_name="sns") as client:
+                        response = await asyncio.wait_for(
+                            client.create_topic(
+                                Name=cls.encode_topic(cls.get_topic_name(topic, context, topic_prefix))
+                            ),
+                            timeout=40,
+                        )
+                        topic_arn = response.get("TopicArn")
+                except (
+                    Exception,
+                    asyncio.TimeoutError,
+                ) as e2:
+                    error_message2 = str(e) if not isinstance(e, asyncio.TimeoutError) else "Network timeout"
+                    logging.getLogger("transport.aws_sns_sqs").warning(
+                        "Unable to create topic [sns] on AWS ({})".format(error_message2)
+                    )
+                    raise AWSSNSSQSException(error_message2, log_level=context.get("log_level")) from e2
+
+                logging.getLogger("transport.aws_sns_sqs").info(
+                    "Already existing SNS topic '{}' has different topic attributes".format(topic_arn)
+                )
+            else:
+                logging.getLogger("transport.aws_sns_sqs").warning(
+                    "Unable to create topic [sns] on AWS ({})".format(error_message)
+                )
+                raise AWSSNSSQSException(error_message, log_level=context.get("log_level")) from e
+
+        if update_attributes and topic_attributes and topic_arn and not overwrite_attributes:
             logging.getLogger("transport.aws_sns_sqs").warning(
-                "Unable to create topic [sns] on AWS ({})".format(error_message)
+                "Will not overwrite existing attributes on SNS topic '{}'".format(topic_arn)
             )
-            raise AWSSNSSQSException(error_message, log_level=context.get("log_level")) from e
+            update_attributes = False
+
+        if update_attributes and topic_attributes and topic_arn:
+            for attribute_name, attribute_value in topic_attributes.items():
+                try:
+                    logging.getLogger("transport.aws_sns_sqs").info(
+                        "Updating '{}' attribute on SNS topic '{}'".format(attribute_name, topic_arn)
+                    )
+                    async with connector("tomodachi.sns", service_name="sns") as client:
+                        await client.set_topic_attributes(
+                            TopicArn=topic_arn,
+                            AttributeName=attribute_name,
+                            AttributeValue=attribute_value,
+                        )
+                except (
+                    botocore.exceptions.NoCredentialsError,
+                    aiohttp.client_exceptions.ClientOSError,
+                    botocore.exceptions.PartialCredentialsError,
+                    botocore.exceptions.ClientError,
+                    asyncio.TimeoutError,
+                ) as e:
+                    error_message = str(e)
+                    logging.getLogger("transport.aws_sns_sqs").warning(
+                        "Unable to create topic [sns] on AWS ({})".format(error_message)
+                    )
+                    raise AWSSNSSQSException(error_message, log_level=context.get("log_level")) from e
 
-        topic_arn = response.get("TopicArn")
         if not topic_arn or not isinstance(topic_arn, str):
             error_message = "Missing ARN in response"
             logging.getLogger("transport.aws_sns_sqs").warning(
@@ -872,10 +1011,81 @@ class AWSSNSSQSTransport(Invoker):
         if not queue_policy or not isinstance(queue_policy, dict):
             raise Exception("SQS policy is invalid")
 
+        config_base = context.get("options", {}).get("aws_sns_sqs", context.get("options", {}).get("aws", {}))
+        aws_config_base = context.get("options", {}).get("aws", {})
+
+        sqs_kms_master_key_id: Optional[str] = None
+        for config_value in (
+            config_base.get("aws_sqs_kms_master_key_id"),
+            config_base.get("sqs_kms_master_key_id"),
+            aws_config_base.get("aws_sqs_kms_master_key_id"),
+            aws_config_base.get("sqs_kms_master_key_id"),
+            aws_config_base.get("aws_kms_master_key_id"),
+            aws_config_base.get("kms_master_key_id"),
+            config_base.get("aws_kms_master_key_id"),
+            config_base.get("kms_master_key_id"),
+        ):
+            if config_value is not None:
+                if isinstance(config_value, str) and config_value == "":
+                    sqs_kms_master_key_id = ""
+                    break
+                elif isinstance(config_value, bool) and config_value is False:
+                    sqs_kms_master_key_id = ""
+                    break
+                elif isinstance(config_value, str) and config_value:
+                    sqs_kms_master_key_id = config_value
+                    break
+                else:
+                    raise ValueError(
+                        "Bad value for aws_sns_sqs option sqs_kms_master_key_id: {}".format(str(config_value))
+                    )
+
+        sqs_kms_data_key_reuse_period_option = (
+            config_base.get(
+                "aws_sqs_kms_data_key_reuse_period",
+                config_base.get(
+                    "sqs_kms_data_key_reuse_period",
+                    config_base.get("aws_kms_data_key_reuse_period", config_base.get("kms_data_key_reuse_period")),
+                ),
+            )
+            or aws_config_base.get(
+                "aws_sqs_kms_data_key_reuse_period",
+                aws_config_base.get(
+                    "sqs_kms_data_key_reuse_period",
+                    aws_config_base.get(
+                        "aws_kms_data_key_reuse_period", aws_config_base.get("kms_data_key_reuse_period")
+                    ),
+                ),
+            )
+            or None
+        )
+        sqs_kms_data_key_reuse_period: Optional[int] = None
+        if sqs_kms_data_key_reuse_period_option is None or sqs_kms_data_key_reuse_period_option is False:
+            sqs_kms_data_key_reuse_period = None
+        if sqs_kms_data_key_reuse_period_option is True:
+            raise ValueError(
+                "Bad value for aws_sns_sqs option sqs_kms_data_key_reuse_period: {}".format(
+                    str(sqs_kms_data_key_reuse_period_option)
+                )
+            )
+        try:
+            if sqs_kms_data_key_reuse_period_option is not None:
+                sqs_kms_data_key_reuse_period = int(sqs_kms_data_key_reuse_period_option)
+            if sqs_kms_data_key_reuse_period == 0:
+                sqs_kms_data_key_reuse_period = None
+        except Exception:
+            raise ValueError(
+                "Bad value for aws_sns_sqs option sqs_kms_data_key_reuse_period: {}".format(
+                    str(sqs_kms_data_key_reuse_period_option)
+                )
+            ) from None
+
         current_queue_attributes = {}
         current_queue_policy = {}
         current_visibility_timeout = None
         current_message_retention_period = None
+        current_kms_master_key_id = None
+        current_kms_data_key_reuse_period_seconds = None
 
         visibility_timeout = None  # not implemented yet
         message_retention_period = None  # not implemented yet
@@ -883,12 +1093,27 @@ class AWSSNSSQSTransport(Invoker):
         try:
             async with connector("tomodachi.sqs", service_name="sqs") as sqs_client:
                 response = await sqs_client.get_queue_attributes(
-                    QueueUrl=queue_url, AttributeNames=["Policy", "VisibilityTimeout", "MessageRetentionPeriod"]
+                    QueueUrl=queue_url,
+                    AttributeNames=[
+                        "Policy",
+                        "VisibilityTimeout",
+                        "MessageRetentionPeriod",
+                        "KmsMasterKeyId",
+                        "KmsDataKeyReusePeriodSeconds",
+                    ],
                 )
                 current_queue_attributes = response.get("Attributes", {})
                 current_queue_policy = json.loads(current_queue_attributes.get("Policy") or "{}")
                 current_visibility_timeout = current_queue_attributes.get("VisibilityTimeout")
+                if current_visibility_timeout:
+                    current_visibility_timeout = int(current_visibility_timeout)
                 current_message_retention_period = current_queue_attributes.get("MessageRetentionPeriod")
+                if current_message_retention_period:
+                    current_message_retention_period = int(current_message_retention_period)
+                current_kms_master_key_id = current_queue_attributes.get("KmsMasterKeyId")
+                current_kms_data_key_reuse_period_seconds = current_queue_attributes.get("KmsDataKeyReusePeriodSeconds")
+                if current_kms_data_key_reuse_period_seconds:
+                    current_kms_data_key_reuse_period_seconds = int(current_kms_data_key_reuse_period_seconds)
         except botocore.exceptions.ClientError:
             pass
 
@@ -900,16 +1125,42 @@ class AWSSNSSQSTransport(Invoker):
             queue_attributes["Policy"] = json.dumps(queue_policy)
 
         if visibility_timeout and current_visibility_timeout and visibility_timeout != current_visibility_timeout:
-            queue_attributes["VisibilityTimeout"] = visibility_timeout  # specified in seconds
+            queue_attributes["VisibilityTimeout"] = str(
+                visibility_timeout
+            )  # SQS.SetQueueAttributes "Attributes" are mapped string -> string
 
         if (
             message_retention_period
             and current_message_retention_period
             and message_retention_period != current_message_retention_period
         ):
-            queue_attributes["MessageRetentionPeriod"] = message_retention_period  # specified in seconds
+            queue_attributes["MessageRetentionPeriod"] = str(
+                message_retention_period
+            )  # SQS.SetQueueAttributes "Attributes" are mapped string -> string
+
+        if (
+            sqs_kms_master_key_id is not None
+            and current_kms_master_key_id != sqs_kms_master_key_id
+            and (current_kms_master_key_id or sqs_kms_master_key_id)
+        ):
+            queue_attributes["KmsMasterKeyId"] = sqs_kms_master_key_id
+
+        if (
+            sqs_kms_data_key_reuse_period is not None
+            and current_kms_data_key_reuse_period_seconds != sqs_kms_data_key_reuse_period
+            and (current_kms_master_key_id or sqs_kms_master_key_id)
+        ):
+            queue_attributes["KmsDataKeyReusePeriodSeconds"] = str(
+                sqs_kms_data_key_reuse_period
+            )  # SQS.SetQueueAttributes "Attributes" are mapped string -> string
 
         if queue_attributes:
+            if current_queue_policy:
+                for attribute_name, _ in queue_attributes.items():
+                    logging.getLogger("transport.aws_sns_sqs").info(
+                        "Updating '{}' attribute on SQS queue '{}'".format(attribute_name, queue_arn)
+                    )
+
             try:
                 async with connector("tomodachi.sqs", service_name="sqs") as sqs_client:
                     response = await sqs_client.set_queue_attributes(QueueUrl=queue_url, Attributes=queue_attributes)
@@ -998,6 +1249,9 @@ class AWSSNSSQSTransport(Invoker):
 
     @classmethod
     async def consume_queue(cls, obj: Any, context: Dict, handler: Callable, queue_url: str) -> None:
+        max_number_of_messages = 10
+        wait_time_seconds = 20
+
         if not connector.get_client("tomodachi.sqs"):
             await cls.create_client("sqs", context)
 
@@ -1033,7 +1287,9 @@ class AWSSNSSQSTransport(Invoker):
                             async with connector("tomodachi.sqs", service_name="sqs") as client:
                                 response = await asyncio.wait_for(
                                     client.receive_message(
-                                        QueueUrl=queue_url, WaitTimeSeconds=20, MaxNumberOfMessages=10
+                                        QueueUrl=queue_url,
+                                        WaitTimeSeconds=wait_time_seconds,
+                                        MaxNumberOfMessages=max_number_of_messages,
                                     ),
                                     timeout=40,
                                 )
@@ -1243,8 +1499,8 @@ class AWSSNSSQSTransport(Invoker):
             )
 
             async def setup_queue(
-                topic: str,
                 func: Callable,
+                topic: Optional[str] = None,
                 queue_name: Optional[str] = None,
                 competing_consumer: Optional[bool] = None,
                 attributes: Optional[Dict[str, Union[str, bool]]] = None,
@@ -1258,18 +1514,19 @@ class AWSSNSSQSTransport(Invoker):
 
                 if queue_name is None:
                     queue_name = cls.get_queue_name(
-                        cls.encode_topic(topic), func.__name__, _uuid, competing_consumer, context
+                        cls.encode_topic(topic or ""), func.__name__, _uuid, competing_consumer, context
                     )
                 else:
                     queue_name = cls.prefix_queue_name(queue_name, context)
 
                 queue_url, queue_arn = await cls.create_queue(queue_name, context)
 
-                if re.search(r"([*#])", topic):
-                    await cls.subscribe_wildcard_topic(topic, queue_arn, queue_url, context, attributes=attributes)
-                else:
-                    topic_arn = await cls.create_topic(topic, context)
-                    await cls.subscribe_topics((topic_arn,), queue_arn, queue_url, context, attributes=attributes)
+                if topic:
+                    if re.search(r"([*#])", topic):
+                        await cls.subscribe_wildcard_topic(topic, queue_arn, queue_url, context, attributes=attributes)
+                    else:
+                        topic_arn = await cls.create_topic(topic, context)
+                        await cls.subscribe_topics((topic_arn,), queue_arn, queue_url, context, attributes=attributes)
 
                 return queue_url
 
@@ -1278,7 +1535,7 @@ class AWSSNSSQSTransport(Invoker):
                     "_aws_sns_sqs_subscribers", []
                 ):
                     queue_url = await setup_queue(
-                        topic, func, queue_name=queue_name, competing_consumer=competing, attributes=attributes
+                        func, topic=topic, queue_name=queue_name, competing_consumer=competing, attributes=attributes
                     )
                     await cls.consume_queue(obj, context, handler, queue_url=queue_url)
             except Exception:
@@ -1295,7 +1552,7 @@ publish = AWSSNSSQSTransport.publish
 
 
 def aws_sns_sqs(
-    topic: str,
+    topic: Optional[str] = None,
     callback_kwargs: Optional[Union[List, Set, Tuple]] = None,
     competing: Optional[bool] = None,
     queue_name: Optional[str] = None,
@@ -1308,7 +1565,7 @@ def aws_sns_sqs(
     return cast(
         Callable,
         __aws_sns_sqs(
-            topic,
+            topic=topic,
             callback_kwargs=callback_kwargs,
             competing=competing,
             queue_name=queue_name,
