@@ -2,8 +2,9 @@ import asyncio
 import datetime
 import inspect
 import logging
+import random
 import time
-from typing import Any, Awaitable, Callable, Dict, List, Optional, Tuple, Union, cast  # noqa
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union, cast
 
 import pytz
 import tzlocal
@@ -20,8 +21,9 @@ from tomodachi.invoker import Invoker
 class Scheduler(Invoker):
     close_waiter = None
 
+    @classmethod
     async def schedule_handler(
-        cls: Any,
+        cls,
         obj: Any,
         context: Dict,
         func: Any,
@@ -38,34 +40,38 @@ class Scheduler(Invoker):
         )
 
         async def handler() -> None:
-            kwargs = dict(original_kwargs)
-
             increase_execution_context_value("scheduled_functions_current_tasks")
-            increase_execution_context_value("scheduled_functions_total_tasks")
             try:
+                kwargs = dict(original_kwargs)
+
+                increase_execution_context_value("scheduled_functions_total_tasks")
+
                 routine = func(*(obj,), **kwargs)
-                if isinstance(routine, Awaitable):
+                if inspect.isawaitable(routine):
                     await routine
-            except Exception as e:
+
+            except (Exception, asyncio.CancelledError) as e:
                 logging.getLogger("exception").exception("Uncaught exception: {}".format(str(e)))
-            decrease_execution_context_value("scheduled_functions_current_tasks")
+            except BaseException as e:
+                logging.getLogger("exception").exception("Uncaught exception: {}".format(str(e)))
+            finally:
+                decrease_execution_context_value("scheduled_functions_current_tasks")
 
         context["_schedule_scheduled_functions"] = context.get("_schedule_scheduled_functions", [])
         context["_schedule_scheduled_functions"].append((interval, timestamp, timezone, immediately, func, handler))
 
-        start_func = cls.start_scheduler(cls, obj, context)
+        start_func = cls.start_scheduler(obj, context)
         return (await start_func) if start_func else None
 
     @classmethod
     def schedule_handler_with_interval(cls, interval: Union[str, int]) -> Callable:
-        def _func(cls: Any, obj: Any, context: Dict, func: Any) -> Any:
-            return cls.schedule_handler(cls, obj, context, func, interval=interval)
+        def _func(_: Any, obj: Any, context: Dict, func: Any) -> Any:
+            return cls.schedule_handler(obj, context, func, interval=interval)
 
         return _func
 
-    @classmethod
+    @staticmethod
     def next_call_at(
-        cls,
         current_time: float,
         interval: Optional[Union[str, int]] = None,
         timestamp: Optional[str] = None,
@@ -168,7 +174,7 @@ class Scheduler(Invoker):
             if isinstance(interval, int):
                 return int(current_time + interval)
 
-            interval_aliases = {
+            interval_aliases: Dict[Tuple[str, ...], Union[str, int]] = {
                 ("every second", "1s", "1 s", "1second", "1 second", "second", "secondly", "once per second"): 1,
                 (
                     "every minute",
@@ -213,7 +219,7 @@ class Scheduler(Invoker):
                 ("sunday", "sundays", "sun", "every sunday", "once per sunday"): "0 0 * * 0",
                 ("weekday", "weekdays", "every weekday"): "0 0 * * 1-5",
                 ("weekend", "weekends", "every weekend"): "0 0 * * 0,6",
-            }  # type: Dict[Tuple[str, ...], Union[str, int]]
+            }
             interval = interval.lower()
 
             if interval.endswith("s") or interval.endswith("seconds"):
@@ -223,7 +229,7 @@ class Scheduler(Invoker):
                     pass
 
             try:
-                interval_value = [v for k, v in interval_aliases.items() if interval in k][0]  # type: Union[str, int]
+                interval_value: Union[str, int] = [v for k, v in interval_aliases.items() if interval in k][0]
             except IndexError:
                 interval_value = interval
             if isinstance(interval_value, int):
@@ -241,9 +247,10 @@ class Scheduler(Invoker):
 
         return int(current_time + 60 * 60 * 24 * 365 * 100)
 
-    def get_timezone(cls: Any, timezone: Optional[str] = None) -> Optional[str]:
+    @staticmethod
+    def get_timezone(timezone: Optional[str] = None) -> Optional[str]:
         if timezone:
-            tz_aliases = {
+            tz_aliases: Dict[Tuple[str, ...], str] = {
                 (
                     "+00:00",
                     "-00:00",
@@ -280,7 +287,7 @@ class Scheduler(Invoker):
                 ("-10:00", "-1000", "GMT -1000", "GMT -10:00", "GMT -10"): "Etc/GMT+10",
                 ("-11:00", "-1100", "GMT -1100", "GMT -11:00", "GMT -11"): "Etc/GMT+11",
                 ("-12:00", "-1200", "GMT -1200", "GMT -12:00", "GMT -12"): "Etc/GMT+12",
-            }  # type: Dict[Tuple[str, ...], str]
+            }
             try:
                 try:
                     timezone = [
@@ -296,8 +303,9 @@ class Scheduler(Invoker):
 
         return timezone
 
+    @classmethod
     async def start_schedule_loop(
-        cls: Any,
+        cls,
         obj: Any,
         context: Dict,
         handler: Callable,
@@ -311,36 +319,100 @@ class Scheduler(Invoker):
 
         if not cls.close_waiter:
             cls.close_waiter = asyncio.Future()
-        stop_waiter = asyncio.Future()  # type: asyncio.Future
-        start_waiter = asyncio.Future()  # type: asyncio.Future
+        stop_waiter: asyncio.Future = asyncio.Future()
+        start_waiter: asyncio.Future = asyncio.Future()
 
         async def schedule_loop() -> None:
-            await start_waiter
+            sleep_task: asyncio.Future
+            current_time = time.time()
+            max_sleep_time = 300
+
+            try:
+                sleep_task = asyncio.ensure_future(asyncio.sleep(10))
+                await asyncio.wait([sleep_task, start_waiter], return_when=asyncio.FIRST_COMPLETED)
+                if not sleep_task.done():
+                    sleep_task.cancel()
+                else:
+                    logging.getLogger("transport.schedule").warning(
+                        "Scheduled loop for function '{}' cannot start yet - start waiter not done for 10 seconds".format(
+                            func.__name__
+                        )
+                    )
+                    sleep_task = asyncio.ensure_future(asyncio.sleep(110))
+                    await asyncio.wait([sleep_task, start_waiter], return_when=asyncio.FIRST_COMPLETED)
+                    if not sleep_task.done():
+                        sleep_task.cancel()
+                    else:
+                        logging.getLogger("transport.schedule").warning(
+                            "Scheduled loop for function '{}' cannot start yet - start waiter not done for 120 seconds".format(
+                                func.__name__
+                            )
+                        )
+                        logging.getLogger("exception").exception("Scheduled loop not started for 120 seconds")
+
+                await asyncio.sleep(0.1)
+                await start_waiter
+
+                if not cls.close_waiter or cls.close_waiter.done():
+                    logging.getLogger("transport.schedule").info(
+                        "Scheduled loop for function '{}' never started before service termination".format(
+                            func.__name__
+                        )
+                    )
+                else:
+                    ts0 = cls.next_call_at(current_time, interval, timestamp, timezone)
+                    for _ in range(10):
+                        ts1 = cls.next_call_at(ts0, interval, timestamp, timezone)
+                        ts2 = cls.next_call_at(ts1, interval, timestamp, timezone)
+                        if int(ts2 - ts1) // 2 < max_sleep_time:
+                            max_sleep_time = int(ts2 - ts1) // 2
+                    max_sleep_time = min(max(max_sleep_time, 10), 300)
+            except (Exception, asyncio.CancelledError) as e:
+                logging.getLogger("exception").exception("Uncaught exception: {}".format(str(e)))
+
+                await asyncio.sleep(0.1)
+                await start_waiter
+
+                if not cls.close_waiter or cls.close_waiter.done():
+                    logging.getLogger("transport.schedule").info(
+                        "Scheduled loop for function '{}' never started before service termination".format(
+                            func.__name__
+                        )
+                    )
 
             next_call_at = None
             prev_call_at = None
-            tasks = []  # type: List
-            current_time = time.time()
+            tasks: List = []
             too_many_tasks = False
             threshold = 20
             run_immediately = immediately
-            sleep_task: asyncio.Future
 
-            while not cls.close_waiter.done():
+            while cls.close_waiter and not cls.close_waiter.done():
                 try:
                     if not run_immediately:
                         last_time = current_time
                         actual_time = time.time()
                         current_time = last_time + 1 if int(last_time + 1) < int(actual_time) else actual_time
+
                         if next_call_at is None:
                             next_call_at = cls.next_call_at(current_time, interval, timestamp, timezone)
                             if prev_call_at and prev_call_at == next_call_at:
+                                if int(last_time + 60) < int(actual_time):
+                                    logging.getLogger("transport.schedule").warning(
+                                        "Scheduled tasks for function '{}' is out of time sync and may not run".format(
+                                            func.__name__
+                                        )
+                                    )
+                                    logging.getLogger("exception").exception("Scheduled task loop out of sync")
+
                                 next_call_at = None
                                 await asyncio.sleep(1)
                                 continue
                         sleep_diff = int(current_time + 1) - actual_time + 0.001
                         if next_call_at > time.time() + 8:
                             sleep_diff = int((next_call_at - time.time()) / 3)
+                        if sleep_diff >= max_sleep_time:
+                            sleep_diff = int(max_sleep_time - random.random() * 5)
                         if sleep_diff >= 2:
                             sleep_task = asyncio.ensure_future(asyncio.sleep(sleep_diff))
                             await asyncio.wait([sleep_task, cls.close_waiter], return_when=asyncio.FIRST_COMPLETED)
@@ -363,7 +435,7 @@ class Scheduler(Invoker):
                         if not too_many_tasks and len(tasks) >= threshold:
                             too_many_tasks = True
                             logging.getLogger("transport.schedule").warning(
-                                'Too many scheduled tasks ({}) for function "{}"'.format(threshold, func.__name__)
+                                "Too many scheduled tasks ({}) for function '{}'".format(threshold, func.__name__)
                             )
                             threshold = threshold * 2
                         await asyncio.sleep(1)
@@ -377,7 +449,7 @@ class Scheduler(Invoker):
                         continue
                     if too_many_tasks and len(tasks) < 15:
                         logging.getLogger("transport.schedule").info(
-                            'Tasks within threshold for function "{}" - resumed'.format(func.__name__)
+                            "Tasks within threshold for function '{}' - resumed".format(func.__name__)
                         )
                         threshold = 20
                     too_many_tasks = False
@@ -391,7 +463,10 @@ class Scheduler(Invoker):
                             )
                         )
                     tasks.append(task)
-                except Exception as e:
+                except (Exception, asyncio.CancelledError) as e:
+                    logging.getLogger("exception").exception("Uncaught exception: {}".format(str(e)))
+                    await asyncio.sleep(1)
+                except BaseException as e:
                     logging.getLogger("exception").exception("Uncaught exception: {}".format(str(e)))
                     await asyncio.sleep(1)
 
@@ -425,12 +500,12 @@ class Scheduler(Invoker):
             if not stop_waiter.done():
                 stop_waiter.set_result(None)
 
-        loop = asyncio.get_event_loop()  # type: Any
+        loop: Any = asyncio.get_event_loop()
 
         stop_method = getattr(obj, "_stop_service", None)
 
         async def stop_service(*args: Any, **kwargs: Any) -> None:
-            if not cls.close_waiter.done():
+            if cls.close_waiter and not cls.close_waiter.done():
                 cls.close_waiter.set_result(None)
 
                 if not start_waiter.done():
@@ -457,7 +532,8 @@ class Scheduler(Invoker):
 
         loop.create_task(schedule_loop())
 
-    async def start_scheduler(cls: Any, obj: Any, context: Dict) -> Optional[Callable]:
+    @classmethod
+    async def start_scheduler(cls, obj: Any, context: Dict) -> Optional[Callable]:
         if context.get("_schedule_loop_started"):
             return None
         context["_schedule_loop_started"] = True
@@ -483,9 +559,7 @@ class Scheduler(Invoker):
             for interval, timestamp, timezone, immediately, func, handler in context.get(
                 "_schedule_scheduled_functions", []
             ):
-                await cls.start_schedule_loop(
-                    cls, obj, context, handler, func, interval, timestamp, timezone, immediately
-                )
+                await cls.start_schedule_loop(obj, context, handler, func, interval, timestamp, timezone, immediately)
 
         return _schedule
 
